@@ -6,12 +6,18 @@ use std::collections::HashMap;
 use std::fs;
 use std::io;
 use std::path::PathBuf;
+use std::sync::Mutex;
 
 /// HTML template rendering engine that should be constructed for once.
 pub struct Dojang {
     /// Mapping between the template file name and the renderer along with the file content.
     templates: HashMap<String, (Executer, String)>,
+
+    /// Map of the registered functions.
     functions: HashMap<String, FunctionContainer>,
+
+    /// Files read from "include". Those are cached here.
+    includes: Mutex<HashMap<String, String>>,
 }
 
 impl Dojang {
@@ -20,6 +26,7 @@ impl Dojang {
         Dojang {
             templates: HashMap::new(),
             functions: HashMap::new(),
+            includes: Mutex::new(HashMap::new()),
         }
     }
 
@@ -56,39 +63,110 @@ impl Dojang {
     /// Adds a function that can be used in the template.
     ///
     /// If there is already an existing function with same name, this will return error.
+    /// Use the appropriate function based on the number of parameters that the function take.
+    /// (e.g for functions taking 2 params, use add_function_2). Functions with 4 params are
+    /// supported at max.
+    ///
+    /// Note that the parameter of the function must be convertible to Value. Supported types are
+    /// String, i64, f64 and boolean.
     ///
     /// # Arguments
     ///
     /// * `function_name` - Name of the function.
-    /// * `function` - The body of the function. Function must be taking `serde_json::Value` as a
-    /// parameters and return `serde_json::Value`. Also, the function must be one of enum values
-    /// defined in `FunctionContainer`, which are categorized by the number of parameters.
-    ///
-    /// For example, if the function takes 2 params, it must be `FunctionContainer::F2`.
+    /// * `function` - The body of the function.
     ///
     /// # Examples
     ///
     /// ```
     /// use serde_json::Value;
-    /// let mut dojang = dojang::Dojang::new();
+    /// use dojang::dojang::Dojang;
     ///
-    /// // Register a function that takes two numeric values and returns the sum of it.
-    /// dojang.add_function("func".to_string(), dojang::FunctionContainer::F2(|a: Value, b: Value| -> Value {
-    ///        Value::Number(serde_json::Number::from(
-    ///            a.as_i64().unwrap() + b.as_i64().unwrap(),
-    ///        ))
-    ///    }));
+    /// fn func(a: i64) -> i64 { a + 1 }
+    /// fn func2(mut a: String, b: String) -> String {
+    ///     a.push_str(&b); a.push_str("hi"); a
+    /// }
+    ///
+    /// let mut dj = Dojang::new();
+    ///
+    /// dj.add_function_1("func".to_string(), func);
+    /// dj.add_function_2("func2".to_string(), func2);
     /// ```
-    pub fn add_function(
+
+    pub fn add_function_1<T: 'static, V: 'static>(
         &mut self,
         function_name: String,
-        function: FunctionContainer,
-    ) -> Result<&Self, String> {
+        function: fn(T) -> V,
+    ) -> Result<&Self, String>
+    where
+        T: From<Operand>,
+        V: Into<Operand>,
+    {
         if self.functions.contains_key(&function_name) {
             return Err(format!("{} is already added as a function", function_name));
         }
 
-        self.functions.insert(function_name, function);
+        self.functions
+            .insert(function_name, to_function_container1(function));
+        Ok(self)
+    }
+
+    pub fn add_function_2<T1: 'static, T2: 'static, V: 'static>(
+        &mut self,
+        function_name: String,
+        function: fn(T1, T2) -> V,
+    ) -> Result<&Self, String>
+    where
+        T1: From<Operand>,
+        T2: From<Operand>,
+        V: Into<Operand>,
+    {
+        if self.functions.contains_key(&function_name) {
+            return Err(format!("{} is already added as a function", function_name));
+        }
+
+        self.functions
+            .insert(function_name, to_function_container2(function));
+        Ok(self)
+    }
+
+    pub fn add_function_3<T1: 'static, T2: 'static, T3: 'static, V: 'static>(
+        &mut self,
+        function_name: String,
+        function: fn(T1, T2, T3) -> V,
+    ) -> Result<&Self, String>
+    where
+        T1: From<Operand>,
+        T2: From<Operand>,
+        T3: From<Operand>,
+        V: Into<Operand>,
+    {
+        if self.functions.contains_key(&function_name) {
+            return Err(format!("{} is already added as a function", function_name));
+        }
+
+        self.functions
+            .insert(function_name, to_function_container3(function));
+        Ok(self)
+    }
+
+    pub fn add_function_4<T1: 'static, T2: 'static, T3: 'static, T4: 'static, V: 'static>(
+        &mut self,
+        function_name: String,
+        function: fn(T1, T2, T3, T4) -> V,
+    ) -> Result<&Self, String>
+    where
+        T1: From<Operand>,
+        T2: From<Operand>,
+        T3: From<Operand>,
+        T4: From<Operand>,
+        V: Into<Operand>,
+    {
+        if self.functions.contains_key(&function_name) {
+            return Err(format!("{} is already added as a function", function_name));
+        }
+
+        self.functions
+            .insert(function_name, to_function_container4(function));
         Ok(self)
     }
 
@@ -111,7 +189,7 @@ impl Dojang {
     /// // Add every files under ./tests as a template.
     /// dojang.load("./tests");
     /// ```
-    pub fn load(&mut self, dir_name: &str) -> Result<&Self, String> {
+    pub fn load(&mut self, dir_name: &str) -> Result<&mut Self, String> {
         match get_all_file_path_under_dir(dir_name) {
             Ok(files) => {
                 for file in files {
@@ -166,9 +244,15 @@ impl Dojang {
     /// // Render 'template_file' with the provided context.
     /// dojang.load("./tests").unwrap().render("template_file", serde_json::from_str(r#"{ "test" : { "title" : "Welcome to Dojang"} }"#).unwrap());
     /// ```
-    pub fn render(&self, file_name: &str, value: Value) -> Result<String, String> {
+    pub fn render(&mut self, file_name: &str, value: Value) -> Result<String, String> {
         if let Some((executer, file_content)) = self.templates.get(&file_name.to_string()) {
-            executer.render(&mut Context::new(value), &self.functions, file_content)
+            executer.render(
+                &mut Context::new(value),
+                &self.templates,
+                &self.functions,
+                file_content,
+                &mut self.includes,
+            )
         } else {
             Err(format!("Template {} is not found", file_name))
         }
@@ -182,6 +266,60 @@ fn get_all_file_path_under_dir(dir_name: &str) -> io::Result<Vec<PathBuf>> {
         .collect()
 }
 
+pub fn to_function_container1<T: 'static + From<Operand>, V: 'static + Into<Operand>>(
+    func: fn(T) -> V,
+) -> FunctionContainer {
+    FunctionContainer::F1(Box::new(move |v: Operand| -> Operand {
+        func(v.into()).into()
+    }))
+}
+
+pub fn to_function_container2<
+    'a,
+    T1: 'static + From<Operand>,
+    T2: 'static + From<Operand>,
+    V: 'static + Into<Operand>,
+>(
+    func: fn(T1, T2) -> V,
+) -> FunctionContainer {
+    FunctionContainer::F2(Box::new(move |v1: Operand, v2: Operand| -> Operand {
+        func(v1.into(), v2.into()).into()
+    }))
+}
+
+pub fn to_function_container3<
+    'a,
+    T1: 'static + From<Operand>,
+    T2: 'static + From<Operand>,
+    T3: 'static + From<Operand>,
+    V: 'static + Into<Operand>,
+>(
+    func: fn(T1, T2, T3) -> V,
+) -> FunctionContainer {
+    FunctionContainer::F3(Box::new(
+        move |v1: Operand, v2: Operand, v3: Operand| -> Operand {
+            func(v1.into(), v2.into(), v3.into()).into()
+        },
+    ))
+}
+
+pub fn to_function_container4<
+    'a,
+    T1: 'static + From<Operand>,
+    T2: 'static + From<Operand>,
+    T3: 'static + From<Operand>,
+    T4: 'static + From<Operand>,
+    V: 'static + Into<Operand>,
+>(
+    func: fn(T1, T2, T3, T4) -> V,
+) -> FunctionContainer {
+    FunctionContainer::F4(Box::new(
+        move |v1: Operand, v2: Operand, v3: Operand, v4: Operand| -> Operand {
+            func(v1.into(), v2.into(), v3.into(), v4.into()).into()
+        },
+    ))
+}
+
 #[test]
 fn render() {
     let template = "<% if a == 1 { %> Hi <% } else { %><%= a %><% } %>".to_string();
@@ -240,3 +378,23 @@ fn render_from_dir() {
 "#
     );
 }
+
+#[cfg(test)]
+fn func(a: i64, b: i64) -> i64 {
+    a + b
+}
+
+#[test]
+fn render_function() {
+    let template = r#"func(a,b) = <%= func(a, b) %>"#.to_string();
+    let mut dojang = Dojang::new();
+    assert!(dojang.add("some_template".to_string(), template).is_ok());
+    assert!(dojang.add_function_2("func".to_string(), func).is_ok());
+
+    assert_eq!(
+        dojang
+            .render("some_template", serde_json::json!({"a" : 1, "b" : 2 }))
+            .unwrap(),
+        "func(a,b) = 3"
+    );
+}
