@@ -1,4 +1,5 @@
 use crate::expr::*;
+use serde_json::Value;
 use std::collections::HashMap;
 
 // Evaluate the parsed expression.
@@ -17,6 +18,9 @@ pub enum Expr {
 pub struct Function {
     pub name: String,
     pub params: Vec<Eval>,
+
+    // Accessor (e.g obj[]) is treated as a function.
+    pub is_accessor: bool,
 }
 
 impl Eval {
@@ -122,9 +126,7 @@ impl Eval {
                             // Then this is the start of the function.
                             let function_tokens =
                                 get_tokens_belong_to_function(&mut expr.ops, inst_index)?;
-                            println!("Func tokens :{:?}", function_tokens);
                             let function = handle_function_tokens(function_tokens)?;
-                            println!("function {:?}", function);
 
                             // Now insert the Operand::Function as a placeholder.
                             expr.ops.insert(
@@ -136,6 +138,20 @@ impl Eval {
                             // replaced by the Expr::Function.
                             function_name_to_function.insert(function.name.clone(), function);
                         }
+                        Some(Op::BracketOpen) | Some(Op::Dot) => {
+                            // Then this is the start of the property accessor ([] or .)
+                            let accessor_tokens =
+                                get_tokens_belong_to_accessor(&mut expr.ops, inst_index)?;
+                            let accessor = handle_accessor_tokens(accessor_tokens)?;
+
+                            // Now insert the Operand::Function as a placeholder.
+                            expr.ops.insert(
+                                inst_index,
+                                Op::Operand(Operand::Function(accessor.name.clone())),
+                            );
+
+                            function_name_to_function.insert(accessor.name.clone(), accessor);
+                        }
                         _ => {}
                     },
                     _ => {}
@@ -198,12 +214,81 @@ fn get_tokens_belong_to_function(
     }
 
     if opened_paren != 0 {
-        return Err(format!("Function does not have closing ): {:?}", ops));
+        return Err(format!("Function does not have closing ')'. {:?}", ops));
     }
 
     Ok(ops.drain(function_start..inst_index + 1).collect())
 }
 
+fn get_tokens_belong_to_accessor(
+    ops: &mut Vec<Op>,
+    mut inst_index: usize,
+) -> Result<Vec<Op>, String> {
+    let function_start = inst_index;
+
+    inst_index += 1;
+
+    let mut opened_paren = 0;
+    'single_property: loop {
+        while inst_index < ops.len() {
+            match ops.get(inst_index).unwrap() {
+                Op::BracketOpen => {
+                    opened_paren += 1;
+                }
+                Op::BracketClose => {
+                    opened_paren -= 1;
+                    if opened_paren == 0 {
+                        break;
+                    }
+                }
+                Op::Dot => {
+                    if opened_paren == 0 {
+                        match ops.get(inst_index + 1) {
+                            Some(Op::Operand(Operand::Object(_))) => {
+                                inst_index += 2;
+                                continue;
+                            }
+                            _ => {
+                                return Err(format!(
+                                    "Only raw string should come after dot, you gave : {:?}",
+                                    ops.get(inst_index + 1)
+                                ));
+                            }
+                        }
+                    }
+                }
+                _ => {
+                    if opened_paren == 0 {
+                        break 'single_property;
+                    }
+                }
+            }
+
+            inst_index += 1;
+        }
+
+        // Property accessor can be chanined.
+        // (e.g obj[...][...][...])
+        // In this case, entire [...][...][...] will be included.
+        match ops.get(inst_index + 1) {
+            Some(Op::BracketOpen) | Some(Op::Dot) => {
+                inst_index += 1;
+                continue;
+            }
+            _ => {
+                inst_index += 1;
+                break;
+            }
+        }
+    }
+
+    if opened_paren != 0 {
+        return Err(format!("Function does not have closing ']'. {:?}", ops));
+    }
+
+    Ok(ops.drain(function_start..inst_index).collect())
+}
+
 fn handle_function_tokens(mut ops: Vec<Op>) -> Result<Function, String> {
     // Should be at least function_name, (, ).
     assert!(ops.len() >= 3);
@@ -262,6 +347,79 @@ fn handle_function_tokens(mut ops: Vec<Op>) -> Result<Function, String> {
     Ok(Function {
         name: function_name,
         params,
+        is_accessor: false,
+    })
+}
+
+fn handle_accessor_tokens(mut ops: Vec<Op>) -> Result<Function, String> {
+    // Should be at least accessor_name, [, ].
+    assert!(ops.len() >= 3);
+
+    let accessor_name = match ops.get(0).unwrap() {
+        Op::Operand(operand) => match operand {
+            Operand::Object(object) => object.name.clone(),
+            _ => panic!("Accessor name is not object."),
+        },
+        _ => panic!("Accessor name is not proper operator."),
+    };
+
+    let mut inst_index = 0;
+    let mut opened_paren = 0;
+    let mut accessors = Vec::new();
+
+    // Remove function name.
+    ops.drain(0..1);
+
+    while inst_index < ops.len() {
+        match ops.get(inst_index).unwrap() {
+            Op::BracketOpen => {
+                opened_paren += 1;
+            }
+            Op::BracketClose => {
+                opened_paren -= 1;
+
+                if opened_paren == 0 {
+                    let tokens = Tokens {
+                        ops: ops.drain(1..inst_index).collect(),
+                    };
+
+                    accessors.push(Eval::new(tokens)?);
+                    inst_index = 0;
+
+                    // Remove '[' and ']'.
+                    ops.drain(0..2);
+
+                    continue;
+                }
+            }
+            Op::Dot => {
+                if opened_paren == 0 {
+                    if let Op::Operand(Operand::Object(object)) = ops.get(inst_index + 1).unwrap() {
+                        accessors.push(Eval {
+                            expr: vec![Expr::Op(Op::Operand(Operand::Value(Value::from(
+                                object.name.clone(),
+                            ))))],
+                        })
+                    }
+
+                    ops.drain(inst_index..inst_index + 2);
+                    continue;
+                }
+            }
+            _ => {}
+        }
+
+        inst_index += 1;
+    }
+
+    if !ops.is_empty() {
+        return Err(format!("Accessor parsing error {:?}", ops));
+    }
+
+    Ok(Function {
+        name: accessor_name,
+        params: accessors,
+        is_accessor: true,
     })
 }
 
@@ -330,7 +488,7 @@ fn create_simple_binary_expr() {
                 name: "some".to_string(),
             })),
             Op::Equal,
-            Op::Operand(Operand::Number(3)),
+            Op::Operand(Operand::Value(Value::from(3))),
         ],
     };
 
@@ -340,7 +498,7 @@ fn create_simple_binary_expr() {
             Expr::Op(Op::Operand(Operand::Object(Object {
                 name: "some".to_string(),
             }))),
-            Expr::Op(Op::Operand(Operand::Number(3))),
+            Expr::Op(Op::Operand(Operand::Value(Value::from(3)))),
         ],
     };
     assert_eq!(Eval::new(expr).unwrap(), expected_eval);
@@ -356,7 +514,7 @@ fn create_binary_with_unary_expr() {
             Op::Equal,
             Op::Not,
             Op::Not,
-            Op::Operand(Operand::Number(3)),
+            Op::Operand(Operand::Value(Value::from(3))),
         ],
     };
 
@@ -368,7 +526,7 @@ fn create_binary_with_unary_expr() {
             }))),
             Expr::Op(Op::Not),
             Expr::Op(Op::Not),
-            Expr::Op(Op::Operand(Operand::Number(3))),
+            Expr::Op(Op::Operand(Operand::Value(Value::from(3)))),
         ],
     };
     assert_eq!(Eval::new(expr).unwrap(), expected_eval);
@@ -500,7 +658,7 @@ fn check_assign_op() {
                 name: "a".to_string(),
             })),
             Op::And,
-            Op::Operand(Operand::Number(3)),
+            Op::Operand(Operand::Value(Value::from(3))),
         ],
     };
 
@@ -514,7 +672,7 @@ fn check_assign_op() {
             Expr::Op(Op::Operand(Operand::Object(Object {
                 name: "a".to_string(),
             }))),
-            Expr::Op(Op::Operand(Operand::Number(3))),
+            Expr::Op(Op::Operand(Operand::Value(Value::from(3)))),
         ],
     };
     assert_eq!(Eval::new(expr).unwrap(), expected_eval);
@@ -585,9 +743,9 @@ fn arithmetic_expression() {
                 name: "f".to_string(),
             })),
             Op::Plus,
-            Op::Operand(Operand::Number(1)),
+            Op::Operand(Operand::Value(Value::from(1))),
             Op::Plus,
-            Op::Operand(Operand::Number(2)),
+            Op::Operand(Operand::Value(Value::from(2))),
         ],
     };
 
@@ -618,8 +776,8 @@ fn arithmetic_expression() {
             Expr::Op(Op::Operand(Operand::Object(Object {
                 name: "f".to_string(),
             }))),
-            Expr::Op(Op::Operand(Operand::Number(1))),
-            Expr::Op(Op::Operand(Operand::Number(2))),
+            Expr::Op(Op::Operand(Operand::Value(Value::from(1)))),
+            Expr::Op(Op::Operand(Operand::Value(Value::from(2)))),
         ],
     };
     assert_eq!(Eval::new(expr).unwrap(), expected_eval);
@@ -633,9 +791,9 @@ fn function_expr() {
                 name: "func".to_string(),
             })),
             Op::ParenOpen,
-            Op::Operand(Operand::Number(1)),
+            Op::Operand(Operand::Value(Value::from(1))),
             Op::Plus,
-            Op::Operand(Operand::Number(1)),
+            Op::Operand(Operand::Value(Value::from(1))),
             Op::Comma,
             Op::Operand(Operand::Object(Object {
                 name: "a".to_string(),
@@ -661,8 +819,8 @@ fn function_expr() {
                     Eval {
                         expr: vec![
                             Expr::Op(Op::Plus),
-                            Expr::Op(Op::Operand(Operand::Number(1))),
-                            Expr::Op(Op::Operand(Operand::Number(1))),
+                            Expr::Op(Op::Operand(Operand::Value(Value::from(1)))),
+                            Expr::Op(Op::Operand(Operand::Value(Value::from(1)))),
                         ],
                     },
                     Eval {
@@ -676,6 +834,7 @@ fn function_expr() {
                         })))],
                     },
                 ],
+                is_accessor: false,
             }),
             Expr::Op(Op::Operand(Operand::Object(Object {
                 name: "x".to_string(),
@@ -684,3 +843,101 @@ fn function_expr() {
     };
     assert_eq!(Eval::new(expr).unwrap(), expected_eval);
 }
+
+#[test]
+fn bracket_expr() {
+    let expr = Tokens {
+        ops: vec![
+            Op::Operand(Operand::Object(Object {
+                name: "func".to_string(),
+            })),
+            Op::ParenOpen,
+            Op::Operand(Operand::Object(Object {
+                name: "a".to_string(),
+            })),
+            Op::BracketOpen,
+            Op::Operand(Operand::Value(Value::from(1))),
+            Op::BracketClose,
+            Op::BracketOpen,
+            Op::Operand(Operand::Value(Value::from(2))),
+            Op::BracketClose,
+            Op::ParenClose,
+            Op::Plus,
+            Op::Operand(Operand::Object(Object {
+                name: "x".to_string(),
+            })),
+        ],
+    };
+
+    let expected_eval = Eval {
+        expr: vec![
+            Expr::Op(Op::Plus),
+            Expr::Function(Function {
+                name: "func".to_string(),
+                params: vec![Eval {
+                    expr: vec![Expr::Function(Function {
+                        name: "a".to_string(),
+                        params: vec![
+                            Eval {
+                                expr: vec![Expr::Op(Op::Operand(Operand::Value(Value::from(1))))],
+                            },
+                            Eval {
+                                expr: vec![Expr::Op(Op::Operand(Operand::Value(Value::from(2))))],
+                            },
+                        ],
+                        is_accessor: true,
+                    })],
+                }],
+                is_accessor: false,
+            }),
+            Expr::Op(Op::Operand(Operand::Object(Object {
+                name: "x".to_string(),
+            }))),
+        ],
+    };
+    assert_eq!(Eval::new(expr).unwrap(), expected_eval);
+}
+
+#[test]
+fn get_tokens_belong_to_accessor_test() {
+    let mut tokens = vec![
+        Op::Operand(Operand::Object(Object {
+            name: "a".to_string(),
+        })),
+        Op::Dot,
+        Op::Operand(Operand::Object(Object {
+            name: "b".to_string(),
+        })),
+        Op::BracketOpen,
+        Op::Operand(Operand::Value(Value::from(123))),
+        Op::BracketClose,
+        Op::Dot,
+        Op::Operand(Operand::Object(Object {
+            name: "c".to_string(),
+        })),
+        Op::Plus,
+        Op::Operand(Operand::Value(Value::from(123))),
+    ];
+
+    let expected = vec![
+        Op::Operand(Operand::Object(Object {
+            name: "a".to_string(),
+        })),
+        Op::Dot,
+        Op::Operand(Operand::Object(Object {
+            name: "b".to_string(),
+        })),
+        Op::BracketOpen,
+        Op::Operand(Operand::Value(Value::from(123))),
+        Op::BracketClose,
+        Op::Dot,
+        Op::Operand(Operand::Object(Object {
+            name: "c".to_string(),
+        })),
+    ];
+
+    assert_eq!(
+        get_tokens_belong_to_accessor(&mut tokens, 0).unwrap(),
+        expected
+    );
+}
