@@ -3,8 +3,9 @@ use crate::eval::*;
 use crate::expr::*;
 use html_escape::encode_safe;
 #[cfg(test)]
-use serde_json::Value;
+use serde_json::{json, Value};
 use std::collections::{BTreeMap, HashMap};
+use std::sync::Mutex;
 
 // The executer that renders the template.
 #[derive(Debug)]
@@ -131,7 +132,10 @@ impl Executer {
         }
 
         if !opened.is_empty() {
-            return Err(format!("No closing bracket found {:?}", insts));
+            return Err(format!(
+                "No closing bracket found {}",
+                pretty_print_insts(insts)
+            ));
         }
 
         // Handle break and continue.
@@ -197,8 +201,10 @@ impl Executer {
     pub fn render(
         &self,
         context: &mut Context,
+        templates: &HashMap<String, (Executer, String)>,
         functions: &HashMap<String, FunctionContainer>,
         template: &str,
+        includes: &mut Mutex<HashMap<String, String>>,
     ) -> Result<String, String> {
         let mut rendered = String::new();
         let mut for_index_counter = HashMap::new();
@@ -212,16 +218,18 @@ impl Executer {
                     }
                     Show::ExprEscaped(eval) => {
                         rendered.push_str(&*encode_safe(&Executer::run_eval(
-                            context, functions, &eval,
+                            context, templates, functions, &eval, includes,
                         )?));
                     }
                     Show::ExprUnescaped(eval) => {
-                        rendered.push_str(&Executer::run_eval(context, functions, &eval)?);
+                        rendered.push_str(&Executer::run_eval(
+                            context, templates, functions, &eval, includes,
+                        )?);
                     }
                 },
                 Action::If(eval) | Action::While(eval) => {
                     // Jump only if the condition is false. Otherwise just go to next instruction.
-                    if !eval.run(context, functions)?.is_true() {
+                    if !eval.run(context, templates, functions, includes)?.is_true() {
                         if let Some(next) = self.jump_table.get(&inst_index) {
                             inst_index = *next;
                             continue;
@@ -275,7 +283,7 @@ impl Executer {
                         continue;
                     }
                     _ => {
-                        &Executer::run_eval(context, functions, &eval)?;
+                        &Executer::run_eval(context, templates, functions, &eval, includes)?;
                     }
                 },
                 Action::Else() => {}
@@ -290,17 +298,42 @@ impl Executer {
     // Runs Eval if it is not empty.
     fn run_eval(
         context: &mut Context,
+        templates: &HashMap<String, (Executer, String)>,
         functions: &HashMap<String, FunctionContainer>,
         eval: &Eval,
+        includes: &mut Mutex<HashMap<String, String>>,
     ) -> Result<String, String> {
         if eval.is_empty() {
             return Ok("".to_string());
         }
 
-        Ok(eval.run(context, functions)?.to_str())
+        Ok(eval.run(context, templates, functions, includes)?.to_str())
     }
 }
 
+fn pretty_print_insts(insts: &Vec<Action<Eval>>) -> String {
+    let mut result = String::new();
+
+    let mut padding = String::new();
+    for action in insts {
+        match action {
+            Action::If(_) | Action::While(_) | Action::For(_) => {
+                result.push_str(&format!("{}{:?} \n", padding, action));
+                padding.push(' ');
+            }
+            Action::End() => {
+                padding.pop();
+                result.push_str(&format!("{}{:?} \n", padding, action));
+            }
+            action => {
+                result.push_str(&format!("{}{:?} \n", padding, action));
+            }
+        }
+    }
+
+    result
+}
+
 fn convert_expr_to_eval(action: Action<Tokens>) -> Result<Action<Eval>, String> {
     match action {
         Action::Show(show) => match show {
@@ -467,6 +500,8 @@ fn test_nested_continue_jump_table() {
 #[test]
 fn test_arithmetic_exec() {
     let context_json = r#"{"a": 1, "b":2, "c": 3, "d" : 2, "e" : 6, "f" : 2}"#;
+    let mut includes = Mutex::new(HashMap::new());
+
     {
         let context_value: Value = serde_json::from_str(context_json).unwrap();
         let mut context = Context::new(context_value);
@@ -475,7 +510,13 @@ fn test_arithmetic_exec() {
         let template = r"<% b = a = (a + b * c - e / d) * f;e=a+b; if e == 16 { g = 5; h = 6} %>g=<%= g %> h=<%= h %>";
         let executer = Executer::new(Parser::parse(template).unwrap()).unwrap();
         let result = executer
-            .render(&mut context, &HashMap::new(), template)
+            .render(
+                &mut context,
+                &HashMap::new(),
+                &HashMap::new(),
+                template,
+                &mut includes,
+            )
             .unwrap();
 
         assert_eq!(result, "g=5 h=6".to_string());
@@ -485,11 +526,18 @@ fn test_arithmetic_exec() {
 #[test]
 fn test_while_exec() {
     let template = r#"<% a = 0; while a < 3 { if a == 1 { %>One <% } else { %>a = <%= a %> <% } a = a + 1 } %>"#;
+    let mut includes = Mutex::new(HashMap::new());
     let executer = Executer::new(Parser::parse(template).unwrap()).unwrap();
 
     let mut context = Context::new(Value::from(""));
     let result = executer
-        .render(&mut context, &HashMap::new(), template)
+        .render(
+            &mut context,
+            &HashMap::new(),
+            &HashMap::new(),
+            template,
+            &mut includes,
+        )
         .unwrap();
 
     assert_eq!(result, "a = 0 One a = 2 ".to_string());
@@ -498,6 +546,7 @@ fn test_while_exec() {
 #[test]
 fn test_for_in_statement() {
     let template = r#"<% for x in arr { %><%= x %> <% } %>"#;
+    let mut includes = Mutex::new(HashMap::new());
     let executer = Executer::new(Parser::parse(template).unwrap()).unwrap();
 
     let context_json = r#"{"arr" : [1,2,3,4,5]}"#;
@@ -505,7 +554,13 @@ fn test_for_in_statement() {
     let mut context = Context::new(context_value);
 
     let result = executer
-        .render(&mut context, &HashMap::new(), template)
+        .render(
+            &mut context,
+            &HashMap::new(),
+            &HashMap::new(),
+            template,
+            &mut includes,
+        )
         .unwrap();
 
     assert_eq!(result, "1 2 3 4 5 ".to_string());
@@ -515,6 +570,7 @@ fn test_for_in_statement() {
 fn test_nested_for_in_statement() {
     let template =
         r#"<% for x in arr { for y in arr2 { %><%= x %>*<%= y %>=<%= x * y %>,<% } } %>"#;
+    let mut includes = Mutex::new(HashMap::new());
     let executer = Executer::new(Parser::parse(template).unwrap()).unwrap();
 
     let context_json = r#"{"arr" : [1,2,3,4,5], "arr2" : [1,2]}"#;
@@ -522,7 +578,13 @@ fn test_nested_for_in_statement() {
     let mut context = Context::new(context_value);
 
     let result = executer
-        .render(&mut context, &HashMap::new(), template)
+        .render(
+            &mut context,
+            &HashMap::new(),
+            &HashMap::new(),
+            template,
+            &mut includes,
+        )
         .unwrap();
     assert_eq!(
         result,
@@ -533,6 +595,7 @@ fn test_nested_for_in_statement() {
 #[test]
 fn test_break() {
     let template = r#"<% for a in v { if a > b { break; } %><%= a %> <% } %>"#;
+    let mut includes = Mutex::new(HashMap::new());
     let executer = Executer::new(Parser::parse(template).unwrap()).unwrap();
 
     let context_json = r#"{"b" : 3, "v" : [1,2,3,4,5]}"#;
@@ -540,7 +603,13 @@ fn test_break() {
     let mut context = Context::new(context_value);
 
     let result = executer
-        .render(&mut context, &HashMap::new(), template)
+        .render(
+            &mut context,
+            &HashMap::new(),
+            &HashMap::new(),
+            template,
+            &mut includes,
+        )
         .unwrap();
     assert_eq!(result, "1 2 3 ".to_string());
 }
@@ -549,6 +618,7 @@ fn test_break() {
 fn test_nested_break() {
     let template =
         r#"<% for a in v { for b in v { if a * b > 10 { break; } %><%= a * b %> <% } } %>"#;
+    let mut includes = Mutex::new(HashMap::new());
     let executer = Executer::new(Parser::parse(template).unwrap()).unwrap();
 
     let context_json = r#"{"v" : [1,2,3,4,5]}"#;
@@ -556,7 +626,13 @@ fn test_nested_break() {
     let mut context = Context::new(context_value);
 
     let result = executer
-        .render(&mut context, &HashMap::new(), template)
+        .render(
+            &mut context,
+            &HashMap::new(),
+            &HashMap::new(),
+            template,
+            &mut includes,
+        )
         .unwrap();
     assert_eq!(result, "1 2 3 4 5 2 4 6 8 10 3 6 9 4 8 5 10 ".to_string());
 }
@@ -564,6 +640,7 @@ fn test_nested_break() {
 #[test]
 fn test_continue() {
     let template = r#"<% for a in v { if a == b { continue; } %><%= a %> <% } %>"#;
+    let mut includes = Mutex::new(HashMap::new());
     let executer = Executer::new(Parser::parse(template).unwrap()).unwrap();
 
     let context_json = r#"{"b" : 3, "v" : [1,2,3,4,5]}"#;
@@ -571,7 +648,13 @@ fn test_continue() {
     let mut context = Context::new(context_value);
 
     let result = executer
-        .render(&mut context, &HashMap::new(), template)
+        .render(
+            &mut context,
+            &HashMap::new(),
+            &HashMap::new(),
+            template,
+            &mut includes,
+        )
         .unwrap();
     assert_eq!(result, "1 2 4 5 ".to_string());
 }
@@ -579,6 +662,7 @@ fn test_continue() {
 #[test]
 fn test_nested_continue() {
     let template = r#"<% for a in v { for b in v { if a == b { continue; } %><%= b %><% } if a == 2 { continue } } %>"#;
+    let mut includes = Mutex::new(HashMap::new());
     let executer = Executer::new(Parser::parse(template).unwrap()).unwrap();
 
     let context_json = r#"{"v" : [1,2,3,4,5]}"#;
@@ -586,7 +670,13 @@ fn test_nested_continue() {
     let mut context = Context::new(context_value);
 
     let result = executer
-        .render(&mut context, &HashMap::new(), template)
+        .render(
+            &mut context,
+            &HashMap::new(),
+            &HashMap::new(),
+            template,
+            &mut includes,
+        )
         .unwrap();
     assert_eq!(result, "23451345124512351234".to_string());
 }
@@ -594,6 +684,7 @@ fn test_nested_continue() {
 #[test]
 fn test_function() {
     let template = r#"<%= func(a, b) %>"#;
+    let mut includes = Mutex::new(HashMap::new());
     let executer = Executer::new(Parser::parse(template).unwrap()).unwrap();
 
     let context_json = r#"{"a" : 10, "b" : 20}"#;
@@ -603,14 +694,30 @@ fn test_function() {
     let mut functions = HashMap::new();
     functions.insert(
         "func".to_string(),
-        FunctionContainer::F2(|a: Value, b: Value| -> Value {
-            Value::Number(serde_json::Number::from(
-                a.as_i64().unwrap() + b.as_i64().unwrap(),
-            ))
-        }),
+        FunctionContainer::F2(Box::new(|a: Operand, b: Operand| -> Operand {
+            let a = match a {
+                Operand::Value(v) => v.as_i64().unwrap(),
+                _ => 0,
+            };
+
+            let b = match b {
+                Operand::Value(v) => v.as_i64().unwrap(),
+                _ => 0,
+            };
+
+            Operand::Value(Value::from(a + b))
+        })),
     );
 
-    let result = executer.render(&mut context, &functions, template).unwrap();
+    let result = executer
+        .render(
+            &mut context,
+            &HashMap::new(),
+            &functions,
+            template,
+            &mut includes,
+        )
+        .unwrap();
     assert_eq!(result, "30".to_string());
 }
 
@@ -618,6 +725,7 @@ fn test_function() {
 fn test_function_complex() {
     // (12 + 10 * 20 - 2 + 1 + 3) * 2
     let template = r#"<%= func(func(2, a), func(1+func2(a, b, 2), 3)) * 2 %>"#;
+    let mut includes = Mutex::new(HashMap::new());
     let executer = Executer::new(Parser::parse(template).unwrap()).unwrap();
 
     let context_json = r#"{"a" : 10, "b" : 20}"#;
@@ -627,23 +735,52 @@ fn test_function_complex() {
     let mut functions = HashMap::new();
     functions.insert(
         "func".to_string(),
-        FunctionContainer::F2(|a: Value, b: Value| -> Value {
-            Value::Number(serde_json::Number::from(
-                a.as_i64().unwrap() + b.as_i64().unwrap(),
-            ))
-        }),
+        FunctionContainer::F2(Box::new(|a: Operand, b: Operand| -> Operand {
+            let a = match a {
+                Operand::Value(v) => v.as_i64().unwrap(),
+                _ => 0,
+            };
+
+            let b = match b {
+                Operand::Value(v) => v.as_i64().unwrap(),
+                _ => 0,
+            };
+
+            Operand::Value(Value::from(a + b))
+        })),
     );
 
     functions.insert(
         "func2".to_string(),
-        FunctionContainer::F3(|a: Value, b: Value, c: Value| -> Value {
-            Value::Number(serde_json::Number::from(
-                a.as_i64().unwrap() * b.as_i64().unwrap() - c.as_i64().unwrap(),
-            ))
-        }),
+        FunctionContainer::F3(Box::new(|a: Operand, b: Operand, c: Operand| -> Operand {
+            let a = match a {
+                Operand::Value(v) => v.as_i64().unwrap(),
+                _ => 0,
+            };
+
+            let b = match b {
+                Operand::Value(v) => v.as_i64().unwrap(),
+                _ => 0,
+            };
+
+            let c = match c {
+                Operand::Value(v) => v.as_i64().unwrap(),
+                _ => 0,
+            };
+
+            Operand::Value(Value::from(a * b - c))
+        })),
     );
 
-    let result = executer.render(&mut context, &functions, template).unwrap();
+    let result = executer
+        .render(
+            &mut context,
+            &HashMap::new(),
+            &functions,
+            template,
+            &mut includes,
+        )
+        .unwrap();
     assert_eq!(result, "428".to_string());
 }
 
@@ -651,6 +788,7 @@ fn test_function_complex() {
 fn test_function_with_statements() {
     let template =
         r#"<%= while func(a, b) < 10 { a = a + 1; %>(<%= a %>, <%= b %>) <% b = b + 1 } %>"#;
+    let mut includes = Mutex::new(HashMap::new());
     let executer = Executer::new(Parser::parse(template).unwrap()).unwrap();
 
     let context_json = r#"{"a" : 1, "b" : 2}"#;
@@ -660,13 +798,222 @@ fn test_function_with_statements() {
     let mut functions = HashMap::new();
     functions.insert(
         "func".to_string(),
-        FunctionContainer::F2(|a: Value, b: Value| -> Value {
-            Value::Number(serde_json::Number::from(
-                a.as_i64().unwrap() + b.as_i64().unwrap(),
-            ))
-        }),
+        FunctionContainer::F2(Box::new(|a: Operand, b: Operand| -> Operand {
+            let a = match a {
+                Operand::Value(v) => v.as_i64().unwrap(),
+                _ => 0,
+            };
+
+            let b = match b {
+                Operand::Value(v) => v.as_i64().unwrap(),
+                _ => 0,
+            };
+
+            Operand::Value(Value::from(a + b))
+        })),
     );
 
-    let result = executer.render(&mut context, &functions, template).unwrap();
+    let result = executer
+        .render(
+            &mut context,
+            &HashMap::new(),
+            &functions,
+            template,
+            &mut includes,
+        )
+        .unwrap();
     assert_eq!(result, "(2, 2) (3, 3) (4, 4) (5, 5) ".to_string());
 }
+
+#[test]
+fn test_function_taking_object() {
+    let template = r#"<%- func(a) %>"#;
+    let mut includes = Mutex::new(HashMap::new());
+    let executer = Executer::new(Parser::parse(template).unwrap()).unwrap();
+
+    let context_value: Value = json!({"a" : {"b" : 1, "c" : 2}});
+    let mut context = Context::new(context_value);
+
+    let mut functions = HashMap::new();
+    functions.insert(
+        "func".to_string(),
+        FunctionContainer::F1(Box::new(|a: Operand| -> Operand {
+            let a = match a {
+                Operand::Value(v) => v,
+                _ => panic!(""),
+            };
+
+            Operand::Value(Value::String(a.to_string()))
+        })),
+    );
+
+    let result = executer
+        .render(
+            &mut context,
+            &HashMap::new(),
+            &functions,
+            template,
+            &mut includes,
+        )
+        .unwrap();
+
+    assert_eq!(result, r#"{"b":1,"c":2}"#.to_string());
+}
+
+#[test]
+fn test_predefined_include_function() {
+    let template = r#"<%- include(a) %>"#;
+    let mut includes = Mutex::new(HashMap::new());
+    let executer = Executer::new(Parser::parse(template).unwrap()).unwrap();
+
+    let context_json = r#"{"a" : "./tests/test_include.html"}"#;
+    let context_value: Value = serde_json::from_str(context_json).unwrap();
+    let mut context = Context::new(context_value);
+
+    let functions = HashMap::new();
+    let result = executer
+        .render(
+            &mut context,
+            &HashMap::new(),
+            &functions,
+            template,
+            &mut includes,
+        )
+        .unwrap();
+
+    assert_eq!(result, "<html>test include</html>\n".to_string());
+}
+
+#[test]
+fn test_predefined_include_template_function() {
+    let template = r#"<%- include_template(a) %>"#;
+    let mut includes = Mutex::new(HashMap::new());
+    let executer = Executer::new(Parser::parse(template).unwrap()).unwrap();
+
+    let context_json = r#"{"a" : "some_template", "title" : "TITLE"}"#;
+    let context_value: Value = serde_json::from_str(context_json).unwrap();
+    let mut context = Context::new(context_value);
+
+    let functions = HashMap::new();
+    let mut templates = HashMap::new();
+
+    // Register "some_template". This will be loaded from include_template function.
+    let some_template = "<html><%= title %></html>".to_string();
+    templates.insert(
+        "some_template".to_string(),
+        (
+            Executer::new(Parser::parse(&some_template).unwrap()).unwrap(),
+            some_template,
+        ),
+    );
+
+    let result = executer
+        .render(
+            &mut context,
+            &templates,
+            &functions,
+            template,
+            &mut includes,
+        )
+        .unwrap();
+
+    assert_eq!(result, "<html>TITLE</html>".to_string());
+}
+
+#[test]
+fn test_accessor() {
+    let template = r#"<%= 10000 + a[x][y + "c"][z] - 3%>"#;
+    let mut includes = Mutex::new(HashMap::new());
+    let executer = Executer::new(Parser::parse(template).unwrap()).unwrap();
+    let context_json = r#"{"a" : { "b" : { "cc" : { "1" : 1234}}}, "x" : "b", "y" : "c", "z" : 1}"#;
+
+    let context_value: Value = serde_json::from_str(context_json).unwrap();
+    let mut context = Context::new(context_value);
+
+    let result = executer
+        .render(
+            &mut context,
+            &HashMap::new(),
+            &HashMap::new(),
+            template,
+            &mut includes,
+        )
+        .unwrap();
+    assert_eq!(result, "11231".to_string());
+}
+
+#[test]
+fn test_accessor_and_function() {
+    let template = r#"<%= a[x][func2(y, "c")+"d"][func(z)] %>"#;
+    let mut includes = Mutex::new(HashMap::new());
+    let executer = Executer::new(Parser::parse(template).unwrap()).unwrap();
+
+    let context_json =
+        r#"{"a" : { "b" : { "ccd" : { "2" : 1234}}}, "x" : "b", "y" : "c", "z" : 1}"#;
+    let context_value: Value = serde_json::from_str(context_json).unwrap();
+    let mut context = Context::new(context_value);
+
+    let mut functions = HashMap::new();
+    functions.insert(
+        "func2".to_string(),
+        FunctionContainer::F2(Box::new(|a: Operand, b: Operand| -> Operand {
+            let mut a = match a {
+                Operand::Value(v) => v.as_str().unwrap().to_string(),
+                _ => "".to_string(),
+            };
+
+            let b = match b {
+                Operand::Value(v) => v.as_str().unwrap().to_string(),
+                _ => "".to_string(),
+            };
+
+            a.push_str(&b);
+            Operand::Value(Value::from(a))
+        })),
+    );
+
+    functions.insert(
+        "func".to_string(),
+        FunctionContainer::F1(Box::new(|a: Operand| -> Operand {
+            let a = match a {
+                Operand::Value(v) => v.as_i64().unwrap(),
+                _ => 0,
+            };
+
+            Operand::Value(Value::from(a + 1))
+        })),
+    );
+
+    let result = executer
+        .render(
+            &mut context,
+            &HashMap::new(),
+            &functions,
+            template,
+            &mut includes,
+        )
+        .unwrap();
+    assert_eq!(result, "1234".to_string());
+}
+
+#[test]
+fn test_accessor_with_dot() {
+    let template = r#"<%= 10000 + a[x].cc[z] - 3%>"#;
+    let mut includes = Mutex::new(HashMap::new());
+    let executer = Executer::new(Parser::parse(template).unwrap()).unwrap();
+    let context_json = r#"{"a" : { "b" : { "cc" : { "1" : 1234}}}, "x" : "b", "y" : "c", "z" : 1}"#;
+
+    let context_value: Value = serde_json::from_str(context_json).unwrap();
+    let mut context = Context::new(context_value);
+
+    let result = executer
+        .render(
+            &mut context,
+            &HashMap::new(),
+            &HashMap::new(),
+            template,
+            &mut includes,
+        )
+        .unwrap();
+    assert_eq!(result, "11231".to_string());
+}
