@@ -1,22 +1,27 @@
+use serde_json::Value;
+
 #[derive(PartialEq, Debug)]
 pub enum Op {
-    Not,        // !
-    Or,         // ||
-    And,        // &&
-    ParenOpen,  // (
-    ParenClose, // )
-    Equal,      // ==
-    NotEqual,   // !=
-    Less,       // <
-    LessEq,     // <=
-    Greater,    // >
-    GreaterEq,  // >=,
-    Assign,     // =
-    Plus,       // +
-    Minus,      // -
-    Multiply,   // *
-    Divide,     // /
-    Comma,      // ,
+    Not,          // !
+    Or,           // ||
+    And,          // &&
+    ParenOpen,    // (
+    ParenClose,   // )
+    BracketOpen,  // [
+    BracketClose, // ]
+    Equal,        // ==
+    NotEqual,     // !=
+    Less,         // <
+    LessEq,       // <=
+    Greater,      // >
+    GreaterEq,    // >=,
+    Assign,       // =
+    Plus,         // +
+    Minus,        // -
+    Multiply,     // *
+    Divide,       // /
+    Comma,        // ,
+    Dot,          // .
     Operand(Operand),
 }
 
@@ -29,9 +34,7 @@ pub enum Keyword {
 
 #[derive(PartialEq, Debug, Clone)]
 pub enum Operand {
-    Literal(String),
-    Number(i64),
-    Decimal(f64),
+    Value(Value),
     Array(Vec<Operand>),
     Object(Object),
     Keyword(Keyword),
@@ -119,26 +122,26 @@ impl Parser {
                     match after_tag.find("%>") {
                         Some(tag_end) => {
                             let tag_to_parse;
-                            match template.chars().nth(tag_start + 2).unwrap() {
-                                '=' => {
+                            match &template[tag_start + 2..tag_start + 3] {
+                                "=" => {
                                     parse_tree.push(Action::Show(Show::ExprEscaped(Tokens {
                                         ops: Vec::new(),
                                     })));
                                     tag_to_parse = &after_tag[1..tag_end];
                                 }
-                                '-' => {
+                                "-" => {
                                     parse_tree.push(Action::Show(Show::ExprUnescaped(Tokens {
                                         ops: Vec::new(),
                                     })));
                                     tag_to_parse = &after_tag[1..tag_end];
                                 }
-                                '#' => {
+                                "#" => {
                                     // Ignore the comment.
                                     template = &after_tag[tag_end + 2..];
                                     index_offset += tag_end + 2;
                                     continue;
                                 }
-                                '%' => {
+                                "%" => {
                                     // Show '%'.
                                     parse_tree.push(Action::Show(Show::Html {
                                         start: tag_start + 2,
@@ -204,7 +207,7 @@ impl Parser {
             // Handle Operators.
             match template[current..current + 1].find(
                 &[
-                    '&', '|', '(', ')', '!', '=', '<', '>', '+', '-', '*', '/', ',',
+                    '&', '|', '(', ')', '!', '=', '<', '>', '+', '-', '*', '/', ',', '[', ']', '.',
                 ][..],
             ) {
                 Some(_) => {
@@ -312,9 +315,9 @@ impl Parser {
         parse_tree
             .last_mut()
             .unwrap()
-            .add_op(Op::Operand(Operand::Literal(
+            .add_op(Op::Operand(Operand::Value(Value::from(
                 template[start + 1..string_literal_end].to_string(),
-            )));
+            ))));
 
         Ok(string_literal_end + 1)
     }
@@ -337,6 +340,8 @@ impl Parser {
             '!' => action.add_op(Op::Not),
             '(' => action.add_op(Op::ParenOpen),
             ')' => action.add_op(Op::ParenClose),
+            '[' => action.add_op(Op::BracketOpen),
+            ']' => action.add_op(Op::BracketClose),
             '<' => action.add_op(Op::Less),
             '>' => action.add_op(Op::Greater),
             '=' => action.add_op(Op::Assign),
@@ -345,6 +350,7 @@ impl Parser {
             '*' => action.add_op(Op::Multiply),
             '/' => action.add_op(Op::Divide),
             ',' => action.add_op(Op::Comma),
+            '.' => action.add_op(Op::Dot),
             c => {
                 return Err(format!(
                     "Unknown operator at '{}', unknown : {}",
@@ -386,11 +392,13 @@ impl Parser {
         let first_char = operand.chars().nth(0).unwrap();
         if first_char.is_ascii_digit() {
             if operand.contains('.') {
-                action.add_op(Op::Operand(Operand::Decimal(
+                action.add_op(Op::Operand(Operand::Value(Value::from(
                     operand.parse().unwrap_or(0.0),
-                )));
+                ))));
             } else {
-                action.add_op(Op::Operand(Operand::Number(operand.parse().unwrap_or(0))));
+                action.add_op(Op::Operand(Operand::Value(Value::from(
+                    operand.parse().unwrap_or(0),
+                ))));
             }
         } else {
             match operand {
@@ -452,6 +460,8 @@ pub fn operator_priority(op: &Op) -> u32 {
         Op::And => 11,
         Op::Or => 12,
         Op::Assign => 14,
+        Op::BracketOpen => 100,
+        Op::BracketClose => 100,
         Op::ParenOpen => 100,
         Op::ParenClose => 100,
         _ => panic!("Oops {:?}", op),
@@ -476,7 +486,7 @@ fn parse_simple_expr_with_binary() {
                     name: "some".to_string(),
                 })),
                 Op::Equal,
-                Op::Operand(Operand::Number(3)),
+                Op::Operand(Operand::Value(Value::from(3))),
             ],
         })],
     };
@@ -550,7 +560,7 @@ fn parse_simple_literal() {
                     name: "some_value".to_string(),
                 })),
                 Op::Equal,
-                Op::Operand(Operand::Literal("abc".to_string())),
+                Op::Operand(Operand::Value(Value::from("abc".to_string()))),
             ],
         })],
     };
@@ -568,7 +578,7 @@ fn parse_literal_with_escape() {
                     name: "some_value".to_string(),
                 })),
                 Op::Equal,
-                Op::Operand(Operand::Literal("a\\\"bc".to_string())),
+                Op::Operand(Operand::Value(Value::from("a\\\"bc".to_string()))),
                 Op::Operand(Operand::Object(Object {
                     name: "abc".to_string(),
                 })),
@@ -669,11 +679,13 @@ fn parse_if_statement() {
                         name: "var3".to_string(),
                     })),
                     Op::Less,
-                    Op::Operand(Operand::Literal("}".to_string())),
+                    Op::Operand(Operand::Value(Value::from("}".to_string()))),
                 ],
             }),
             Action::Do(Tokens {
-                ops: vec![Op::Operand(Operand::Literal("hello".to_string()))],
+                ops: vec![Op::Operand(Operand::Value(Value::from(
+                    "hello".to_string(),
+                )))],
             }),
             Action::End(),
         ],
@@ -748,7 +760,7 @@ fn parse_if_else_statement() {
                 ],
             }),
             Action::Do(Tokens {
-                ops: vec![Op::Operand(Operand::Literal("b".to_string()))],
+                ops: vec![Op::Operand(Operand::Value(Value::from("b".to_string())))],
             }),
             Action::End(),
             Action::Else(),
@@ -758,16 +770,16 @@ fn parse_if_else_statement() {
                         name: "var1".to_string(),
                     })),
                     Op::Equal,
-                    Op::Operand(Operand::Number(3)),
+                    Op::Operand(Operand::Value(Value::from(3))),
                 ],
             }),
             Action::Do(Tokens {
-                ops: vec![Op::Operand(Operand::Literal("a".to_string()))],
+                ops: vec![Op::Operand(Operand::Value(Value::from("a".to_string())))],
             }),
             Action::End(),
             Action::Else(),
             Action::Do(Tokens {
-                ops: vec![Op::Operand(Operand::Literal("c".to_string()))],
+                ops: vec![Op::Operand(Operand::Value(Value::from("c".to_string())))],
             }),
             Action::End(),
         ],
@@ -794,7 +806,9 @@ fn parse_while_statement() {
                 ],
             }),
             Action::Do(Tokens {
-                ops: vec![Op::Operand(Operand::Literal("hello".to_string()))],
+                ops: vec![Op::Operand(Operand::Value(Value::from(
+                    "hello".to_string(),
+                )))],
             }),
             Action::End(),
         ],
@@ -821,7 +835,9 @@ fn parse_for_statement() {
                 ],
             }),
             Action::Do(Tokens {
-                ops: vec![Op::Operand(Operand::Literal("hello".to_string()))],
+                ops: vec![Op::Operand(Operand::Value(Value::from(
+                    "hello".to_string(),
+                )))],
             }),
             Action::End(),
         ],
@@ -840,9 +856,9 @@ fn parse_function() {
                     name: "func".to_string(),
                 })),
                 Op::ParenOpen,
-                Op::Operand(Operand::Number(1)),
+                Op::Operand(Operand::Value(Value::from(1))),
                 Op::Plus,
-                Op::Operand(Operand::Number(1)),
+                Op::Operand(Operand::Value(Value::from(1))),
                 Op::Comma,
                 Op::Operand(Operand::Object(Object {
                     name: "a".to_string(),
@@ -863,6 +879,66 @@ fn parse_function() {
     assert_eq!(result.unwrap(), expected_expr);
 }
 
+#[test]
+fn parse_braket() {
+    let result = Parser::parse(r#"<% func(a[1][2]) + x %>"#);
+    let expected_expr = Parser {
+        parse_tree: vec![Action::Do(Tokens {
+            ops: vec![
+                Op::Operand(Operand::Object(Object {
+                    name: "func".to_string(),
+                })),
+                Op::ParenOpen,
+                Op::Operand(Operand::Object(Object {
+                    name: "a".to_string(),
+                })),
+                Op::BracketOpen,
+                Op::Operand(Operand::Value(Value::from(1))),
+                Op::BracketClose,
+                Op::BracketOpen,
+                Op::Operand(Operand::Value(Value::from(2))),
+                Op::BracketClose,
+                Op::ParenClose,
+                Op::Plus,
+                Op::Operand(Operand::Object(Object {
+                    name: "x".to_string(),
+                })),
+            ],
+        })],
+    };
+
+    assert_eq!(result.unwrap(), expected_expr);
+}
+
+#[test]
+fn parse_property_accessor() {
+    let result = Parser::parse(r#"<% 1 + a.b[123].c %>"#);
+    let expected_expr = Parser {
+        parse_tree: vec![Action::Do(Tokens {
+            ops: vec![
+                Op::Operand(Operand::Value(Value::from(1))),
+                Op::Plus,
+                Op::Operand(Operand::Object(Object {
+                    name: "a".to_string(),
+                })),
+                Op::Dot,
+                Op::Operand(Operand::Object(Object {
+                    name: "b".to_string(),
+                })),
+                Op::BracketOpen,
+                Op::Operand(Operand::Value(Value::from(123))),
+                Op::BracketClose,
+                Op::Dot,
+                Op::Operand(Operand::Object(Object {
+                    name: "c".to_string(),
+                })),
+            ],
+        })],
+    };
+
+    assert_eq!(result.unwrap(), expected_expr);
+}
+
 #[test]
 fn parse_tags_simple() {
     let template = r#"<html><% if a { %>some tag<% } %><body></body></html>"#;
